package gmodel;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

import riso.numerical.LBFGS;

public class CrfTrainer {
	
	private static double[] derivative;
	private static double[] diag;//Diagonal matrix for LBFGS
	
	public static void init(){
		derivative=new double[Global.numFeatures];
		diag=new double[Global.numFeatures];
	}
	
	private static float getDerivative(int featureId){
        //System.out.println("Getting derivative");
		float sentenceExp=0;
		for (int i=0;i<Global.inputData.size();i++){//Go over all sentences
			sentenceExp+=Global.inputData.get(i).getExpectation(featureId);
		}
		return (float) (Global.getEmpericalExpectation(featureId) -	sentenceExp - Global.weight[featureId]*Global.C);
	}
	
	private static void calculateDerivativeVector(){
        //System.out.println("Calculating derivative vector..");
		for (int featureId=0;featureId<Global.numFeatures;featureId++){//Loop over all features
			derivative[featureId]= getDerivative(featureId);		
		}
        //System.out.println("Calculating derivative vector....done!");
	}

	public static void train(){
		double f;
		double eps, xtol;
		int[] iprint, iflag = new int[1];

		int icall;
		int numCorrections;
		iprint = new int[2];

		numCorrections = 7;
		iprint[0] = -1;
		iprint[1] = 0;
		eps = 1e-1;
		xtol = 1.0 / Double.MAX_VALUE;
		icall = 0;
		iflag[0] = 0;

		do
		{
			f = -Global.getLogLikelihood();//getErrorFunction();
			//System.out.println("Log Likelihood: " + f);
			calculateDerivativeVector();	
			try
			{
				LBFGS.lbfgs(Global.numFeatures, numCorrections, Global.weight, f, derivative, false, diag, iprint, eps, xtol, iflag);
				//w = new Matrix(w_parameter).transpose();

			}
			catch (LBFGS.ExceptionWithIflag e)
			{
				//System.err.println("lbfgs done :)\n" + e);
				System.out.println("Done.");								
				printWeights("out.txt");
				return;
			}
			icall += 1;
			System.out.println("LBFGS iteration: " + icall + "(" + (float)f + ")");
			
		}
		while (iflag[0] != 0 && icall <= 400);
		
		System.out.println("LBFGS Converged!!!");
		printWeights("out.txt");
		
	} 	
	
	public static void printWeights(String fname) {
		try {
			BufferedWriter outFile= new BufferedWriter(new FileWriter("out.txt"));
			
			for (int i=0;i<Global.numFeatures;i++)
				outFile.write(Global.weight[i] + " ");
			
			outFile.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
}
