package gmodel;

//Class for Forward Backward algorithm
public class FBAlgo {
	private int numLabels;//B,I,O etc
	private int numVariables;//Words in sentence?
	
	private float[][] alpha;//scores
	private float[][] beta;
	
	private int START_LABEL=-1;
	private int END_LABEL=-2;
	
	private Sentence currentSentence;//stores reference to the current sentence
	
	public void init(int nLabels, int nVars, int nFeatures){
		numLabels=nLabels;
		numVariables=nVars;

		alpha=new float[numVariables][numLabels];
		beta=new float[numVariables][numLabels];
	}
	
	public void setSentence(Sentence s){
		currentSentence=s;
	}
	
	//returns potential for a given feature for the current sentence
	float phi(int position, int prevLabel, int currLabel ){
                //System.out.println("In phi...");
		float retval=0;			
		for (int i=0;i<Global.numFeatures; i++){
			retval+=Global.weight[i]*currentSentence.evaluateFeature(i, position, prevLabel,currLabel);
		}
		return (float)(Math.exp(retval));			
               //System.out.println("phi done!!!");
	}
	
	//call this function after running forward backward algo to get the scaling factor 
	public float zLambda(){
		float retval=0;
		for (int i=0;i<numLabels;i++){
			retval+=beta[0][i]; //FIXME;
			//retval+=beta[0][i]*phi(0,START_LABEL,i);
		}
		return retval;
	}
	
	float getAlpha(int var, int label){
		float retval=0;
		//System.out.println("In getAlpha....");
		if (var==0){//first word in sentence
			retval= 1;//phi(var,START_LABEL,label);
		}else{
			for (int i=0;i<numLabels;i++){
				retval+=alpha[var-1][i]*phi(var,i,label);
			}
		}
		//System.out.println("getAlpha done!");
		return retval;
	}
	
	float getBeta(int var, int label){
		//System.out.println("In getBeta....");
		float retval=0;
		
		if (var>=numVariables-1){//last variable and above
			retval= 1;//phi(var,label,END_LABEL);                    
		}else{
			for (int i=0;i<numLabels;i++){
				retval+=beta[var+1][i]*phi(var,label,i);
			}
		}
		//System.out.println("getBeta done!!!");
		return retval;
	}
	
	void runAlgorithm(){
		//forward phase
		for (int i=0;i<numVariables;i++){
			for (int j=0;j<numLabels;j++){
				alpha[i][j]=getAlpha(i,j);
			}
		}
		//backward phase
		for (int i=numVariables-1;i>=0;i--){
			for (int j=0;j<numLabels;j++){
				beta[i][j]=getBeta(i,j);
			}
		}	
	}

	public float getExpectation(int featureId)
	{
		float retval=0;
		float term;

		int _xcount=0;
//		System.out.println("Getting model expectation for feature "+featureId);

		for (int j=0;j<currentSentence.length;j++)
		{//WORD J
			int featureFire=currentSentence.evaluateFeature(featureId, j, 0,0);
			if (featureFire!=0)
			{
				float tmpRetVal=0;
				for (int s1=0;s1<numLabels;s1++)
				{//previous label
					for (int s2=0;s2<numLabels;s2++)
					{//current label
						//System.out.println("Calculating term...");
						_xcount++;
						term= 	getAlpha(j, s1) * phi(j,s1,s2) * getBeta(j+1, s2);
						//System.out.println("Term calculated!!!");
						tmpRetVal+=term;
					}
				}
				retval+= (featureFire*tmpRetVal);
			}
		}
		//if (retval>0)
		//	System.out.println("Getting model expectation, retval="+retval+ "("+_xcount+" iterations, phew!)");

		return retval/zLambda();
	}
        
        
}
