package gmodel;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;

public class Global {
	public static String NPtag[];
	public static float C;
		
	private static Hashtable<String, Integer> featureHash;
	private static HashSet<String> dictionary;
	private static String[] featureArray;	
	public static ArrayList<Sentence> inputData;
	
	public static int numFeatures;
	public static double[] weight;//weight vector	

	static{//static init for the class
		numFeatures=0;
		C=1.2f;
	}
	
	public static float getEmpericalExpectation(int featureId){
		float retval=featureHash.get(featureArray[featureId]).floatValue()/numFeatures;
        //System.out.println("Empirical expectation of feature "+ featureArray[featureId] + " --> " + retval );
		return retval;
	}
	
	private static void readSentences(String fname){
		inputData= new ArrayList<Sentence>();
		String currentSentence="";

	    try {
		    BufferedReader br = new BufferedReader(new FileReader(fname));
	
		    String currentWord;
			while ((currentWord = br.readLine()) != null){
				//System.out.println(currentWord);
				
				if (currentWord.equals("")){//End of sentence
					//System.out.println("");
					inputData.add(new Sentence(currentSentence));
                                        System.out.println("Sentence done!!!");
					currentSentence="";//reset the current input sentence
				}else{
					currentSentence=currentSentence.concat(currentWord.concat("\n"));
				}
			}
		
			if (!currentSentence.equals("")){//Last sentence should be added to list
				inputData.add(new Sentence(currentSentence));
                                System.out.println("Last Sentence done!!!");
			}
			br.close();
				
		} catch (IOException e) {
			e.printStackTrace();
		}
	    
	}

	private static void readFeatures(String fname){
		featureHash=new Hashtable<String, Integer>();
		
		try {
		    BufferedReader br = new BufferedReader(new FileReader(fname));

		    String currentLine;
		    String[] featureSplit;
		    //Read the keys into a hashtable
		    while ((currentLine = br.readLine()) != null){
				featureSplit=currentLine.split("[|]");
                                //System.out.println("putting "+featureSplit[0]+ " into hash");
				featureHash.put(featureSplit[0], Integer.parseInt(featureSplit[1]));
			}
			br.close();

			//Make a sequential array of featuresKeys
			featureArray=new String[featureHash.size()];			
			Enumeration<String> e= featureHash.keys();

			int count=0;//Fill the featureKeys into the array
			while (e.hasMoreElements()){
				featureArray[count]=e.nextElement(); //e.toString();
				count++;
			}
			numFeatures=count;
			weight= new double[numFeatures];
			for (int i=0;i<numFeatures;i++)
				weight[i]=0;
			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private static void readDictionary(String fname){
		dictionary=new HashSet<String>();
		
		try {
		    BufferedReader br = new BufferedReader(new FileReader(fname));
		    String currentLine;
		    while ((currentLine = br.readLine()) != null){
		    	dictionary.add(currentLine);
		    }		    
		}catch(IOException e){
			e.printStackTrace();
		}
	}
        
        public static void readWeightFile(String fname)
        {
            try 
            {
                BufferedReader br = new BufferedReader(new FileReader(fname));
                String currentLine= br.readLine();
                
                String[] wts=currentLine.split(" ");
                
                for(int i=0; i<wts.length;i++)
                {
                    weight[i]=Double.parseDouble(wts[i]);
                    //System.out.print(wts[i]+" ");
                }               
                		    
            }
            catch(IOException e)
            {
		e.printStackTrace();
            }
        }
	
	public static void init(String featureFile, String dictFile, String sentenceFile, String wtFile)
        {
		NPtag=new String[4];
		NPtag[0]="S";
		NPtag[1]="B";
		NPtag[2]="I";
		NPtag[3]="O";
		
		readDictionary(dictFile);
                System.out.println("Dictionary Read...");
		readFeatures(featureFile);
                System.out.println(Global.numFeatures + " features Read...");
                if(wtFile==null)
                {
                    readSentences(sentenceFile);
                    System.out.println("Sentences Read...");
                }
                else
                {
                    readWeightFile(wtFile);
                }
	}
	
	public static String getFeatureKey(int featureId){
		return featureArray[featureId];
	}
	
	public static String inDictionary(String word){
		if (dictionary.contains(word))
			return "1";
		else
			return "0";
	}
	
	public static float weightNorm()
	{
		float retval=0;
		for (int i=0;i<numFeatures;i++)
		{
			retval+=weight[i]*weight[i];
			//System.out.print(weight[i] + " ");
		}
		//System.out.println();
		return retval;
	}
	
	public static double getLogLikelihood()
	{
		float bigSum=0;
		for(Sentence s : inputData)
		{
			float sum=0;
			for(int j=0; j<s.length; j++)
			{
				for(int i=0; i<numFeatures;i++)          					
					sum+=weight[i]*s.evaluateFeature(i,j,0,0);                        
			}
			//System.out.print(sum + ", ");
			bigSum+=Math.log(Math.exp(sum)/s.getzLambda());
		}
		//System.out.println();
		return bigSum - weightNorm()*C/2;
	}
}
