package gmodel;

public class Sentence {
	private String words[];
	private String POStags[];
	private String NPtags[];
	private FBAlgo fbalgo;
	
	public int length;
		
	public Sentence(){
		length=0;
	}
	
	public Sentence(String data){
		initSentence(data);
	}
	
	public void initSentence(String data){
		//System.out.println(data);
		String[] lines=data.split("\\n");
		
		length=lines.length;
		
		words	=new String[lines.length+1];
		POStags	=new String[lines.length+1];
		NPtags	=new String[lines.length+1];
		
		words[0]	="";
		POStags[0]	=".";
		NPtags[0]	="S";
		
		//System.out.println("Length: " + length);
		
		for (int i=0;i<lines.length;i++){
			//System.out.println(lines[i]);
			String[] tokens=lines[i].split(" ");
			words[i+1]	=tokens[0];
			POStags[i+1]=tokens[1];
			if (tokens[2].equals("B-NP"))				
				NPtags[i+1]	="B";
			else if (tokens[2].equals("I-NP"))
				NPtags[i+1]	="I";
			else
				NPtags[i+1]	="O";
			
		}
		
		fbalgo=new FBAlgo();
		fbalgo.init(Global.NPtag.length, length, Global.numFeatures);
		fbalgo.setSentence(this);
		fbalgo.runAlgorithm();
	}
	
	public int evaluateFeature(int featureId, int position, int prevLabel,int currLabel){
                //TESTED: takes no time
                //System.out.println("In evaluate feature.....");
		position++;//The calling functions are not aware of position 0 being an extra word.
		
		if (position>=words.length)
			return 0;
		String featureKey=Global.getFeatureKey(featureId);
		char fType=featureKey.charAt(0);
		String myFeature="";
		String prev_curr_posn=NPtags[position]+"/"+NPtags[position-1]+" "+Integer.toString(position)+" ";		
		//String prev_curr_posn=NPtags[position].concat("/".concat(NPtags[position-1].concat(" ".concat(Integer.toString(position)))));
		switch (fType)
		{
		case '0':
			//0 prev_l/curr_l posn InDict
			myFeature="0 "+prev_curr_posn+Global.inDictionary(words[position]);
			//myFeature="0 ".concat(prev_curr_posn.concat(" ".concat(Global.inDictionary(words[position]))));			
			break;
		case '1':
			//1 prev_l/curr_l posn tag
			myFeature="1 "+prev_curr_posn+POStags[position];
			//myFeature="1 ".concat(prev_curr_posn.concat(" ".concat(POStags[position])));			
			break;
		case '2':
			//2 prev_l/curr_l posn bitag
			myFeature="2 "+prev_curr_posn+POStags[position]+"/"+POStags[position-1];			
			//myFeature="2 ".concat(prev_curr_posn.concat(" ".concat(POStags[position].concat("/".concat(POStags[position-1])))));
			break;
		}
                //System.out.println("evaluate feature done");
		//System.out.println(myFeature + " --> " + featureKey);
		if (featureKey.equals(myFeature))
			return 1;
		else
			return 0;
	}
	
	float getExpectation(int featureId){
//                System.out.println("Getting expecation of feature "+featureId);
		return fbalgo.getExpectation(featureId);
		//return (Global.getEmpericalExpectation(featureId) - fbalgo.getExpectation(featureId) - Global.weight[featureId]*0.5f); 
	}
	
	public float getzLambda()
	{
		return fbalgo.zLambda();
	}	
}
